/*
 *      L I S T _ A f t e r
 *
 *  This function is called when it is required
 *  to insert a new node after a specifed node.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 *
 *	14 May 99	DJW	  -	Updated for duplciate handling
 */

#include <liblist.h>
#include "liblistx.h"

int     LIST_After (list_t list, node_t newnode, node_t oldnode)
{
    listclass_t listclass;
    _listmaster_t listmaster;
	int		duplicates;
	int		reply;

    LIBDEBUG ("LIST_After", \
            "'list' parameter is not a valid list pointer", \
            _LIST_ListCheck(list))
    LIBDEBUG ("LIST_After", \
            "'newnode' parameter is not a valid member of this list type",\
            _LIST_NodeCheck(list,newnode))
    LIBDEBUG ("LIST_After", \
            "'newnode is already a member of this list", \
             LIST_Index(list,newnode) == 0)

    listclass = list->listclass;
    listmaster = listclass->master;
	duplicates = listclass->duplicates;

    /*
     *  There is a special case of
     *  'oldnode' being NULL which
     *  means insert at end of list.
     */
    if (oldnode == NULL)
    {
        oldnode = listmaster->last(list);
    }

    LIBDEBUG ("LIST_After", \
              "'oldnode is not a member of this list", \
              LIST_Index(list,oldnode) != 0)

    /*
     *  First see if this is an ordered list
     *  as then we need to do some checks on
     *  whether it is valid to insert at
     *  this point.
     */
    if (listclass->compare)
    {
        if (oldnode != NULL)
        {
            node_t   tempnode;

            reply = LIST_Compare(list,newnode,oldnode);
			if (reply < 0)
            {
                return (LIST_ERROR_SEQUENCE);
            }
			if (reply == 0 && duplicates == LIST_DUPLICATES_NONE)
			{
				return LIST_ERROR_DUPLICATE;
			}
		
            tempnode = LIST_Next(list, oldnode);
            if (tempnode != NULL)
            {
                reply = LIST_Compare(list,newnode,tempnode);
				if (reply > 0)
                {
                    return (LIST_ERROR_SEQUENCE);
                }
				if (reply == 0 && duplicates == LIST_DUPLICATES_NONE)
				{
					return LIST_ERROR_DUPLICATE;
				}
            }
        }
    }
    /*
     *  either we passed checks or list is unordered.
     *  Call type specific support routine to add node.
     */
    newnode = _LIST_REALNODE(list,newnode);
    if (oldnode != NULL)
    {
        oldnode = _LIST_REALNODE(list,oldnode);
    }
    return (listmaster->after(list,newnode,oldnode));
}
