/*
 *      L I S T _ A p p e n d
 *
 *  This function is called when it is required
 *  to insert a new node at the end of a list.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 *
 *  09 May 99   DJW   - Made sure that 'last' node passed in to internal
 *						'After' routine.
 */

#include <liblist.h>
#include "liblistx.h"

int     LIST_Append (list_t list, node_t node)
{

	listclass_t  listclass;
	_listmaster_t listmaster;
	node_t	oldnode;

    LIBDEBUG ("LIST_Append", \
            "'list' parameter is not a valid list pointer", \
            _LIST_ListCheck(list))
    LIBDEBUG ("LIST_Append", \
            "'node' parameter is not a valid member of this list type",\
            _LIST_NodeCheck(list,node))
    LIBDEBUG ("LIST_Append", \
            "'node is already a member of this list", \
             LIST_Index(list,node) == 0)

	listclass = list->listclass;
	listmaster = listclass->master;

	oldnode = listmaster->last(list);
    if (listclass->compare != NULL)
	{
		if (listclass->compare (node, oldnode) < 0)
		{
			return LIST_ERROR_SEQUENCE;
		}
	}
	if (oldnode != NULL)
	{
		oldnode = _LIST_REALNODE(list,oldnode);
	}

	return listmaster->after(list,_LIST_REALNODE(list,node), oldnode);
}

