/*
 *      L I S T _ C l e a r
 *
 *  This function is called when it is required
 *  to destroy all nodes in a list, but leave the
 *  list itself intact.
 *
 *
 * (c) Copyright 1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */

#include <liblist.h>
#include "liblistx.h"


extern
int  LIST_Clear  (list_t list)
{
    register _listmaster_t listmaster;
    node_t  listnode;
    int     result;

    LIBDEBUG ("LIST_Clear", \
           "list does not point to a valid list instance", \
           _LIST_ListCheck(list))

		   listmaster = list->listclass->master;
    for (result = LIST_ERROR_NONE;
			LIST_ERROR_NONE == result && NULL != (listnode = listmaster->first(list)) ; )
    {
		/*
         *  Remove the node from the list
         *  (this cannot fail ?)
         */
        result = listmaster->remove(list, _LIST_REALNODE(list,listnode));

        LIBASSERT (result == LIST_ERROR_NONE);
        /*
         *  Kill off the node
         *  Check no error as if so we had better
         *  abandon before catastrophes happen.
         */
		if (LIST_ERROR_NONE == result)
		{
			result = LIST_FreeNode (list,&listnode);
        }
    }
    return result;
}

