/*
 *      L I S T _ D e l e t e
 *
 *  This function is called when it is required
 *  to delete node (i.e. remove and free it).
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  24 May 96   DJW   - First version
 *
 *  14 July 96  DJW   - Enhanced to support list tracking
 *
 *	16 May 99	DJW	  -	Enhanced to trap errors on the 'remove' stage.
 */


#include <liblist.h>
#include "liblistx.h"


int     LIST_Delete (list_t list, node_t *pnode)
{
	int		reply;

	if (pnode == NULL)
	{
		return LIST_ERROR_BADNODE;
	}
    LIBDEBUG ("LIST_Delete", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListCheck(list));
    LIBDEBUG ("LIST_Delete", \
           "Check that node is of correct type for this list", \
           _LIST_NodeCheck(list,*pnode));
    LIBDEBUG ("LIST_Delete", \
           "Check that node really is in list", \
            LIST_Index(list,*pnode) != 0);

    reply = list->listclass->master->remove(list,_LIST_REALNODE(list,*pnode));
	if (LIST_ERROR_NONE == reply)
	{
		reply = LIST_FreeNode(list,pnode);
	}
	return reply;
}

