/*
 *	   L I S T _ f r e e _ c
 *
 *	   This module is part of the LIBLIST library.
 *
 *	   It frees all nodes in a list, but does not
 *     actually destory the main list.
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *	   Permission to copy and/or distribute is granted under the
 *	   following conditions:
 *
 *	   1). This notice must remain intact.
 *	   2). The author is not responsible for the consequences of use
 *		   this software, no matter how awful, even if they arise
 *		   from defects in it.
 *	   3). Altered versions must not be represented as being the
 *		   original software.
 */

#include "liblistx.h"


/*===================================================== LIST_FREE */
extern
int		LIST_Free (list_t list)
/*		~~~~~~~~~
 *----------------------------------------------------------------*/
{
    node_t  listnode;
    register listclass_t  listclass;
    register _listmaster_t listmaster;

	if (list == NULL)
	{
		return LIST_ERROR_BADLIST;
	}
    listclass = list->listclass;
    listmaster = listclass->master;
    while ((listnode = listmaster->first(list)) != NULL)
    {
        int     result;
        /*
         *  Remove the node from the list
         *  (this cannot fail ?)
         */
        result = listmaster->remove(list, _LIST_REALNODE(list,listnode));
        LIBASSERT (result == LIST_ERROR_NONE);
        /*
         *  Kill off the node
         */
        result = LIST_FreeNode (list,&listnode);
        /*
         *  Check no error as if so we had better
         *  abandon before catastrophes happen.
         */
        if (result != LIST_ERROR_NONE)
        {
            return result;
        }
    }
	return LIST_ERROR_NONE;
}

