/*
 *      _ L I S T _ L i s t C h e c k
 *
 *  This function is an internal support routine that
 *  is used to check that the list parameter seems 
 *  It is normally called mainly from code that is
 *  only present in the debug version of the library,
 *  but there are a couple of other points at which
 *  it is used for safety reasons.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  25 May 96   DJW   - First version
 *
 *  30 Jun 96   DJW   - Enhanced the checks.  Also can now be called from
 *                      within standard version of the library as well as
 *                      (more frequently) the debug version.
 */


#include <liblist.h>
#include "liblistx.h"


extern
int    _LIST_ListCheck   (list_t list)
{
    register listclass_t listclass;
    register _listmaster_t listmaster;

	/*
	 *	List must not be NULL
	 */
    if (list == NULL)
    {
        return 0;
    }

	/*
	 *	list class must not be NULL
	 */
    if ((listclass = list->listclass) == NULL)
    {
        return 0;
    }

	/*
	 *	list master must not be NULL
	 */
    if ((listmaster = listclass->master) == NULL)
    {
        return 0;
    }

	/*
	 *	Size of user node must not be NULL

	 */
    if (listclass->sizeuser == 0)
    {
        return 0;
    }
	/*
	 *	The type must be one we recognise
	 */
    switch (listmaster->listtype)
    {
        case LIST_TYPE_SINGLE:
        case LIST_TYPE_DOUBLE:
        case LIST_TYPE_FIFO:
        case LIST_TYPE_LIFO:
        case LIST_TYPE_BTREE:
        case LIST_TYPE_BALTREE:
        case LIST_TYPE_HASH:
		case LIST_TYPE_EMBEDDED:
				break;
        default:
                return 0;
    }

	/*
	 *	Return the type as our answer.
	 */
    return listmaster->listtype;
}


