/*
 *		L I S T _ M e r g e
 *
 *	This function is called when it is required
 *	to merge two lists.
 *
 *
 * (c) Copyright 1996-2000 David J. Walker
 *	   Email:  dave.walker@icl.com
 *
 *	   Permission to copy and/or distribute granted under the
 *	   following conditions:
 *
 *	   1). This notice must remain intact.
 *	   2). The author is not responsible for the consequences of use
 *		   this software, no matter how awful, even if they
 *		   arise from defects in it.
 *	   3). Altered version must not be represented as being the
 *		   original software.
 *
 *
 *	AMENDMENT HISTORY
 *	~~~~~~~~~~~~~~~~~
 *	07 July 00	DJW   - First version
 *
 */

#include <liblist.h>
#include "liblistx.h"

int 	LIST_Merge (list_t sourcelist, list_t targetlist)
{
	long	count;

	LIBDEBUG ("LIST_Merge", \
			"'sourcelist' parameter is not a valid list pointer", \
			_LIST_ListCheck(sourcelist))
	LIBDEBUG ("LIST_Merge", \
			"'targetlist' parameter is not a valid list pointer", \
			_LIST_ListCheck(targetlist))

	/*
	 *	Check lists of same type - but NOT same list
	 */

	/*
	 *	Now go through the merge process
	 */
	count = LIST_Count (sourcelist);
	while (count--)
	{
		register	node_t	node;
		register	int reply;

		node = LIST_First (sourcelist);
		LIBASSERT(node != NULL);
		reply = LIST_Remove (sourcelist, node);
		LIBASSERT (0 == reply);
		if (reply != 0)
		{
			return reply;
		}
		reply = LIST_Add (targetlist, node);
		if (LIST_ERROR_NONE != reply)
		{
			return reply;
		}
	}

	LIBASSERT (0 == LIST_Count(sourcelist));

	return LIST_ERROR_NONE;
}
